import { NodeSizeEnum } from '../../index';

export class NodeConfiguration {
    static height: number;
    static width: number;
    static collapsibleButtonSize = 5;
    textRect: MindFusion.Drawing.Rect;
    barRect: MindFusion.Drawing.Rect;
    imageRect: MindFusion.Drawing.Rect;
    incomingImageRect: MindFusion.Drawing.Rect;
    outgoingImageRect: MindFusion.Drawing.Rect;
    incomingTextRect: MindFusion.Drawing.Rect;
    outgoingTextRect: MindFusion.Drawing.Rect;
    boundText: MindFusion.Drawing.Rect;

    static SetSize(nodeSize: NodeSizeEnum) {
        switch (nodeSize) {
            case NodeSizeEnum.SMALL:
                NodeConfiguration.height = 15;
                NodeConfiguration.width = NodeConfiguration.height * 3;
                break;
            case NodeSizeEnum.MEDIUM:
                NodeConfiguration.height = 25;
                NodeConfiguration.width = NodeConfiguration.height * 3;
                NodeConfiguration.collapsibleButtonSize = 7;
                break;
            case NodeSizeEnum.LARGE:
                NodeConfiguration.height = 35;
                NodeConfiguration.width = NodeConfiguration.height * 3;
                NodeConfiguration.collapsibleButtonSize = 10;
                break;
        }
    }

    constructor(x, y, nodeSize: NodeSizeEnum) {
        NodeConfiguration.SetSize(nodeSize);
        this.calculateTextPosistion(x, y, nodeSize);
        this.calculateBarPosition(x, y);
        this.calculateImagePosition(x, y, nodeSize);
        if (NodeSizeEnum.SMALL !== nodeSize) {
            this.calculateIncomingLinksImagePosition(x, y);
            this.calculateOutgoingLinksImagePosition(x, y);
            this.calculateIncomingTextPosition(x, y);
            this.calculateOutgoingTextPosition(x, y);
        }

    }

    calculateTextPosistion(x: number, y: number, nodeSize: NodeSizeEnum) {
        x = x + Math.abs(NodeConfiguration.height / 1.5);
        y = y + Math.abs(NodeConfiguration.height / 3);
        this.textRect = new MindFusion.Drawing.Rect(x, y, 8, 8);
        let height = 8;
        if (nodeSize === NodeSizeEnum.LARGE) {
            height = 12;
        }
        this.boundText = new MindFusion.Drawing.Rect(x, y, NodeConfiguration.width - NodeConfiguration.height, height);
    }

    calculateBarPosition(x: number, y: number) {
        const height = NodeConfiguration.height - 4;
        let width = 4;
        if (NodeConfiguration.height < 25) {
            width = 2;
        }
        this.barRect = new MindFusion.Drawing.Rect(x + 2, y + 2, height, width);
    }

    calculateImagePosition(x: number, y: number, nodeSize: NodeSizeEnum) {
        x = x + Math.round(NodeConfiguration.height / 4);
        y = y + Math.round(NodeConfiguration.width / 10);
        const height = Math.abs(NodeConfiguration.height / 2.5);
        const width = height;
        this.imageRect = new MindFusion.Drawing.Rect(x, y, height, width);
    }

    calculateIncomingTextPosition(x: number, y: number) {
        x = x + NodeConfiguration.height;
        y = y + NodeConfiguration.height - 6;
        this.incomingTextRect = new MindFusion.Drawing.Rect(x, y, 8, 8);
    }

    calculateIncomingLinksImagePosition(x: number, y: number) {
        x = x + NodeConfiguration.height + 8;
        y = y + NodeConfiguration.height - 10;
        this.incomingImageRect = new MindFusion.Drawing.Rect(x, y, 8, 8);
    }

    calculateOutgoingLinksImagePosition(x: number, y: number) {
        x = x + NodeConfiguration.height + 16;
        y = y + NodeConfiguration.height - 10;
        this.outgoingImageRect = new MindFusion.Drawing.Rect(x, y, 8, 8);
    }

    calculateOutgoingTextPosition(x: number, y: number) {
        x = x + NodeConfiguration.height + 24;
        y = y + NodeConfiguration.height - 6;
        this.outgoingTextRect = new MindFusion.Drawing.Rect(x, y, 8, 8);
    }
}
