import { NodeAttributeModel, NodeSizeEnum, NodeConfiguration } from '../../index';

import Diagram = MindFusion.Diagramming.Diagram;
import HandlesStyle = MindFusion.Diagramming.HandlesStyle;
import Rect = MindFusion.Drawing.Rect;
import shape = MindFusion.Diagramming.Shape;
import Font = MindFusion.Drawing.Font;
import CoantainerNode = MindFusion.Diagramming.ContainerNode;
import TableNode = MindFusion.Diagramming.TableNode;
import Alignment = MindFusion.Diagramming.Alignment;
import Text = MindFusion.Drawing.Text;

export class NodeHelper {
    container: any;
    private _i = 0;

    constructor(private factory: MindFusion.Diagramming.Factory, private nodeAttribute: NodeAttributeModel, private height: number) {
        this.nodeAttribute = nodeAttribute;
    }
    /**
     * Create the container node
     * @param x
     * @param y
     */
    createContainerNode(x: number, y: number, vo: any) {
        if (this.nodeAttribute.type === NodeSizeEnum.SMALL) {
            this.nodeAttribute.height = NodeConfiguration.height;
            this.height = NodeConfiguration.height;
        }
        this.container = this.factory.createContainerNode(x, y, this.nodeAttribute.width, this.nodeAttribute.height);
        this.container['data'] = vo;
        this.updateContainerVisuals();
        if (this.nodeAttribute.type !== NodeSizeEnum.SMALL && this.nodeAttribute.height < this.height) {
            this.container.setFoldable(true);
            this.container.setFoldIconSize(this.nodeAttribute.collapsibleButtonSize);
        }
        this.container.setCaptionHeight(this.nodeAttribute.height);
        const rect = this.container.getBounds();
        rect.height = this.height;
        this.container.setBounds(rect);
    }
    /**
     * Add nodes to container
     * @param x
     * @param y
     * @param columnText
     * @param columnTextColor
     * @param columnImageSrcUrl
     * @param vo
     */
    addRow(x: number, y: number, columnText: string, columnTextColor: string, columnImageSrcUrl: string, vo: any) {
        const row = this.factory.createShapeNode(x, y + this.nodeAttribute.height + this._i, this.nodeAttribute.width, 10);
        row.setStroke(this.nodeAttribute.barColor);
        const image = this.factory.createShapeNode(x + 4, y + this.nodeAttribute.height + 2 + this._i, 5, 5);
        image.setBrush(this.nodeAttribute.brushColor);
        image.setStroke(this.nodeAttribute.penColor);
        image.setImageLocation(columnImageSrcUrl);
        image.setLocked(true);
        this.container.add(image);
        row.setText(columnText);
        row.setBounds(new Rect(x, y + this.nodeAttribute.height + this._i, this.nodeAttribute.width, 10));
        row.setBrush(this.nodeAttribute.backgroundColor);
        this._i += 10;
        row.setTextColor(columnTextColor);
        row['data'] = vo;
        this.container.add(row);
    }
    /**
     * Attach created node to conatiner
     * @param node
     */
    private attachNode(node: MindFusion.Diagramming.ShapeNode) {
        node.attachTo(this.container);
    }
    /**
     * Overrides the container UpdateVisuals
     */
    private updateContainerVisuals() {
        this.container.onUpdateVisuals = (item: MindFusion.Diagramming.DiagramNode) => {
            const rect = item.bounds;
            rect.width = this.nodeAttribute.width;
            item.setBounds(rect);
            item.setStroke(this.nodeAttribute.barColor);
            const config = new NodeConfiguration(rect.x, rect.y, this.nodeAttribute.type);

            const rectPath = new MindFusion.Drawing.Path();
            rectPath.setPen(this.nodeAttribute.barColor);
            rectPath.addRect(config.barRect.x, config.barRect.y, config.barRect.height, config.barRect.width);
            rectPath.setPen(this.nodeAttribute.barColor);
            rectPath.setBrush(this.nodeAttribute.barColor);
            item.getGraphicsContent().push(rectPath);

            const nodeText = new MindFusion.Drawing.Text(this.nodeAttribute.headingText, config.textRect);
            item.setTooltip(this.nodeAttribute.headingText);
            nodeText.setBounds(config.boundText);
            nodeText.pen = this.nodeAttribute.headingColor;
            nodeText.fitInBounds = true;
            nodeText.setFont(new MindFusion.Drawing.Font(this.nodeAttribute.fontName, 4, false, true, false));
            item.getGraphicsContent().push(nodeText);

            if (this.nodeAttribute.imageUrl !== undefined && this.nodeAttribute.imageUrl !== null && this.nodeAttribute.imageUrl !== '') {
                const image = new MindFusion.Drawing.Image(config.imageRect);
                image.image.src = this.nodeAttribute.imageUrl;
                image.loaded = true;
                item.getGraphicsContent().push(image);
            }
            if (this.nodeAttribute.showIncomingOutgoingLink && this.nodeAttribute.type !== NodeSizeEnum.SMALL) {
                this.createIncomingOutgoingLink(item, rect, config);
            }
        };
    }
    /**
     * Creates Incoming and outgoing links
     * @param item
     * @param rect
     */
    createIncomingOutgoingLink(item: MindFusion.Diagramming.DiagramItem, rect: Rect, config: NodeConfiguration) {
        let nodeText = new MindFusion.Drawing.Text(this.nodeAttribute.incomingLinks.toString(), config.incomingTextRect);
        nodeText.setFont(new MindFusion.Drawing.Font(this.nodeAttribute.fontName, 4, false, true, false));
        item.getGraphicsContent().push(nodeText);
        nodeText = new MindFusion.Drawing.Text(this.nodeAttribute.outgoingLinks.toString(), config.outgoingTextRect);
        nodeText.setFont(new MindFusion.Drawing.Font(this.nodeAttribute.fontName, 4, false, true, false));
        item.getGraphicsContent().push(nodeText);
        let image = new MindFusion.Drawing.Image(config.incomingImageRect);
        image.image.src = this.nodeAttribute.incomingLinkImg;
        image.loaded = true;
        item.getGraphicsContent().push(image);
        image = new MindFusion.Drawing.Image(config.outgoingImageRect);
        image.image.src = this.nodeAttribute.outgoingLinkImg;
        image.loaded = true;
        item.getGraphicsContent().push(image);
    }
}
